﻿package com.cb.utils.simplepaint
{
	import com.cb.utils.simplepaint.brush.PaintBrush;
	import com.cb.utils.simplepaint.brush.strokes.BrushStroke;
	import com.cb.utils.simplepaint.canvas.PaintCanvas;
	
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	
	public class SimplePaint extends Sprite
	{
		public static const SMALL_BRUSH:String = "SMALL_BRUSH";
		public static const LARGE_BRUSH:String = "LARGE_BRUSH";
		public static const PEN_BRUSH:String = "PEN_BRUSH";
		public static const ERASER_BRUSH:String = "ERASER_BRUSH";
		
		private var _height:uint = 400;
		private var _width:uint = 400;
		
		private var _background:Sprite;
		private var _canvas:PaintCanvas;
		private var _brush:PaintBrush;
		private var _brushes:Array;
		private var _brushStyle:String = LARGE_BRUSH;
		private var _ratio:Number = 1;
		
		private var _backgroundColor:uint = 0xFF8F48;
		
		public function SimplePaint(width:int = 400, height:int = 400)
		{			
			_width = width;
			_height = height;
			_background = new Sprite();
			_background.graphics.beginFill(_backgroundColor);
			_background.graphics.drawRect(0,0,width, height);

			_canvas = new PaintCanvas(width, height);
			_brush = new PaintBrush(width, height);
			_brush.addEventListener(BrushStroke.STROKE_TIME_OUT, onStrokeTimeOut);
			
			this.addChild(_background);
			this.addChild(_canvas);	
			this.addChild(_brush);
		}
		
		public function set ratio(ratio:Number):void
		{
			_ratio = ratio;
		}
		
		public function set brushStyle(bs:String):void
		{
			_brushStyle = bs;
		}
		
		public function onStrokeTimeOut(e:Event):void
		{
			brushUp((e.target as BrushStroke).id);
		}
		
		public function setDimensions(width:int, height:int):void
		{
			_background.width = width;
			_background.height = height;
			_brush.setDimensions(width, height);
		}
		
		public function undo():void
		{
			_canvas.undo();
		}
		
		public function clear():void
		{
			_canvas.clear();
		}
		
		public function set background(img:Bitmap):void
		{
			if(img.width > _width){
				img.width = _width;
			}
			if(img.height > _height){
				img.height = _height;
			}
			_background.addChild(img);
		}
		
		public function brushDown(id:String, pt:Point):void
		{
			brushUp(id);
			_brush.beginPaint(id, pt, _brushStyle, _ratio);
			_brush.brushMove(id, pt);
		}
		
		public function brushMove(id:String, pt:Point):void
		{
			_brush.brushMove(id, pt);
		}
		
		public function brushUp(id:String):void
		{
			if(_brush.getStroke(id)){
				_canvas.addStroke(_brush.getStroke(id));
			}
			_brush.endPaint(id);
		}
	}
}