package com.cb.utils.simplepaint.brush.strokes
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.geom.ColorTransform;
	import flash.geom.Matrix;
	import flash.geom.Point;

	/**
	 * Base class for all brush strokes that use the paint algorithm.
	 * 
	 */

	public class PaintStroke extends BrushStroke
	{
		protected var _stroke:Shape;
		protected var _wrapper:Sprite;
		protected var _bitmapData:BitmapData;
		protected var _bitmap:Bitmap;
		protected var _width:int;
		protected var _height:int;
		protected var _ratio:int;
		
		public function PaintStroke(color:uint, start:Point, id:String, width:int, height:int, ratio:Number = 1)
		{
			super(color, start, id);
			_width = width;
			_height = height;
			_ratio = ratio;
		}
		
		protected function create():void
		{
			var colorTransform:ColorTransform = _stroke.transform.colorTransform;
			colorTransform.color = _color;
			_stroke.transform.colorTransform = colorTransform;
			
			_wrapper = new Sprite();
			//_wrapper.cacheAsBitmapMatrix = new Matrix();
			_wrapper.addChild(_stroke);
			
			_bitmapData = new BitmapData(_width, _height, true, 0x00FF3300);
			_bitmap = new Bitmap(_bitmapData);
			//_bitmap.cacheAsBitmapMatrix = new Matrix();
			this.addChild(_bitmap);
		}
		
		override public function paint(point:Point):void
		{
			super.paint(point);
			var startx:int = _point.x;
			var starty:int = _point.y;
			var endx:int = point.x;
			var endy:int = point.y;
			
			var t:int;
			var distance:int;
			var xerr:int = 0;
			var yerr:int = 0;
			var delta_x:int;
			var delta_y:int;
		    
			var incx:int;
			var incy:int;
		
			/* compute the distances in both directions */
			delta_x=endx-startx;
			delta_y=endy-starty;
		
			/* Compute the direction of the increment,
			   an increment of 0 means either a horizontal or vertical
			   line.
			*/
			if(delta_x>0) incx=1;
			else if(delta_x==0) incx=0;
			else incx=-1;
		
			if(delta_y>0) incy=1;
			else if(delta_y==0) incy=0;
			else incy=-1;
		
			/* determine which distance is greater */
			delta_x=Math.abs(delta_x);
			delta_y=Math.abs(delta_y);
			if(delta_x>delta_y) distance=delta_x;
			else distance=delta_y;
		
			/* draw the line */
			for(t=0; t<=distance+1; t++) {
				wp(startx, starty);
		       
				xerr+=delta_x;
				yerr+=delta_y;
				if(xerr>distance) {
					xerr-=distance;
					startx+=incx;
				}
				if(yerr>distance) {
					yerr-=distance;
					starty+=incy;
				}
			}
			_point = point;	
		}
		
		protected function wp(startx:int, starty:int):void
		{
			_wrapper.x = startx - (_wrapper.width / 2);
			_wrapper.y = starty - (_wrapper.height / 2);
			
			var matrix:Matrix = new Matrix();
			matrix.tx = _wrapper.x;
			matrix.ty = _wrapper.y;
			
			_bitmapData.draw(_wrapper, matrix);
		}
	}
}